/**
 * Pancake
 *
 * A simple, fast, self-hosted invoicing application
 *
 * @package     Pancake
 * @author      Pancake Dev Team
 * @copyright   Copyright (c) 2015, Pancake Payments
 * @license     https://www.pancakeapp.com/license
 * @link        https://www.pancakeapp.com
 * @since       Version 4.8.29
 */

(function () {
    "use strict";

    var Cache = {
        settings: {
            import: {
                $items: null,
                line_items_showing: 0,
                line_items_count: 0
            }
        },
        invoices: {
            reminders: {
                reminder_checkboxes: null
            }
        }

    };

    var Helpers = {
        show_error: function(title, message) {
            if (!title) {
                title = __("error:title");
            }

            if (!message) {
                message = __("error:subtitle");
            }

            open_reveal("<div class='modal-error'><h3>"+title+"</h3><p>"+message+"</p></div>");
        }
    };

    var Pancake = {
        common: {
            init: function () {

            },
            finalize: function () {

            }
        },
        invoices: {
            reminders: function(cache) {
                var $reminders = $("#reminders");

                cache.reminder_checkboxes = $reminders.find('[type="checkbox"]');

                $reminders.find("th:first").on("click", function check_all_reminders() {
                    var count_checked = cache.reminder_checkboxes.filter(":checked").length;

                    if (cache.reminder_checkboxes.length == count_checked) {
                        cache.reminder_checkboxes.prop('checked', false);
                    } else {
                        cache.reminder_checkboxes.prop('checked', true);
                    }
                }).on("click", "td:first-child", function check_reminder(event) {
                    if (!$(event.target).is('input')) {
                        $(event.target).find('input').prop('checked', function (i, val) {
                            return !val;
                        });
                    }
                });
            }
        },
        settings: {
            index: function () {
                $('.upgrade-btn').on('click', function upgrade_pancake(event) {

                    var url = $(this).attr('href');

                    event.preventDefault();
                    $(this).addClass('disabled').html($(this).data('loading-text'));
                    $.get(url).done(function (data) {
                        if (data == "UPDATED" || url.indexOf("check_latest_version") !== -1) {
                            window.location.reload(true);
                        } else {
                            // This will replace the page with the right error page if there is an error with the update.
                            var newDoc = document.open("text/html", "replace");
                            newDoc.write(data);
                            newDoc.close();
                        }
                    }).fail(function (jqXHR, textStatus) {
                        Helpers.show_error(null, textStatus);
                    });
                });
            },
            import: function (cache) {
                cache.$items = $(".js-item");
                cache.line_items_showing = parseInt(cache.$items.not(".hidden").last().attr("class").split("js-item").join("").split("-")[1]);
                cache.line_items_count = parseInt(cache.$items.last().attr("class").split("js-item").join("").split("-")[1]);
                
                $(".js-show-another-item").on("click", function show_another_item(event) {
                    event.preventDefault();

                    if (cache.line_items_showing >= cache.line_items_count) {
                        // Don't do anything; there are no more items to include.
                        return;
                    }

                    if (cache.line_items_showing >= (cache.line_items_count - 1)) {
                        // Hide the button, as after this click it won't work again.
                        $(this).hide();
                    }

                    var new_item = '.js-item-' + (cache.line_items_showing + 1);
                    cache.$items.filter(new_item).removeClass("hidden");
                    cache.line_items_showing += 1;
                });
            }
        }
    };

    window.Cache = Cache;
    window.Pancake = Pancake;
}());


UTIL = {
    fire: function (func, funcname) {
        var namespace = Pancake, args = null;
        funcname = (funcname === undefined) ? 'init' : funcname;

        if (Cache[func] !== undefined) {
            if (Cache[func][funcname] !== undefined) {
                args = Cache[func][funcname];
            } else {
                args = Cache[func];
            }
        }

        if (func !== '' && namespace[func] && typeof namespace[func][funcname] == 'function') {
            namespace[func][funcname](args);
        }
    },
    loadEvents: function () {
        var bodyId = document.body.id;
        UTIL.fire('common');
        $.each(document.body.className.split(/\s+/), function (i, classnm) {
            // Ignore classes that are not for JS to understand.
            if (classnm.indexOf("controller-") !== -1 || classnm.indexOf("module-") !== -1 || classnm.indexOf("action-") !== -1 || classnm == "body-wrap" || classnm == "not-login-layout" || classnm == "main-layout" || classnm == "admin" || classnm == "not-admin") {
                return;
            }

            UTIL.fire(classnm);
            UTIL.fire(classnm, bodyId);
        });
        UTIL.fire('common', 'finalize');
    }
};

UTIL.loadEvents();